#include <zephyr.h>
#include <device.h>
#include <soc.h>
#include <devicetree.h>
#include <drivers/gpio.h>
#include <drivers/uart.h>


/* 1000 msec = 1 sec */
#define SLEEP_TIME_MS 1000

/* The devicetree node identifier for the "led0" alias. */
#define LED0_NODE DT_ALIAS(led0)

#define LED0 DT_GPIO_LABEL(LED0_NODE, gpios)
#define PIN0 DT_GPIO_PIN(LED0_NODE, gpios)
#define FLAGS0 DT_GPIO_FLAGS(LED0_NODE, gpios)

#define LED1_NODE DT_ALIAS(led1)
#define LED1 DT_GPIO_LABEL(LED1_NODE, gpios)
#define PIN1 DT_GPIO_PIN(LED1_NODE, gpios)
#define FLAGS1 DT_GPIO_FLAGS(LED1_NODE, gpios)

#define UART0_NODE DT_NODELABEL(uart0)
#define UART0 DT_LABEL(UART0_NODE)
#define UART_LABEL		DT_LABEL(DT_NODELABEL(uart0))


void main(void)
{
	const struct device *green_led_handle;
	const struct device *red_led_handle;
	const struct device *uart_0_handle;
	bool led_is_on = true;
	volatile int ret;
	const uint8_t message[] = "Hello World! \n\r";

	green_led_handle = device_get_binding(LED0);
	if (green_led_handle == NULL)
	{
		return;
	}

	red_led_handle = device_get_binding(LED1);
	if (red_led_handle == NULL)
	{
		return;
	}

	uart_0_handle = device_get_binding(UART_LABEL);

	if (!uart_0_handle) {
		return;
	}

	ret = gpio_pin_configure(green_led_handle, PIN0, GPIO_OUTPUT_ACTIVE | FLAGS0);
	if (ret < 0)
	{
		return;
	}

	ret = gpio_pin_configure(red_led_handle, PIN1, GPIO_OUTPUT_ACTIVE | FLAGS1);
	if (ret < 0)
	{
		return;
	}

	while (1)
	{
		gpio_pin_set(green_led_handle, PIN0, (int)led_is_on);
		led_is_on = !led_is_on;
		gpio_pin_set(red_led_handle, PIN1, (int)led_is_on);
		ret = uart_tx(uart_0_handle, message, sizeof(message), 100);
		k_msleep(SLEEP_TIME_MS);
	}
}